@@ -14,7 +14,6 @@
 # limitations under the License.
 
 import mock
-import six
 
 from metalsmith import _instance
 from metalsmith.test import test_provisioner
@@ -125,27 +124,26 @@ class TestInstanceStates(test_provisioner.Base):
     def test_to_dict(self, mock_ips):
         self.node.provision_state = 'wait call-back'
         self.node.to_dict.return_value = {'node': 'dict'}
-        self.node.instance_info = {'metalsmith_hostname': 'host'}
         mock_ips.return_value = {'private': ['1.2.3.4']}
 
         to_dict = self.instance.to_dict()
         self.assertEqual({'allocation': None,
-                          'hostname': 'host',
+                          'hostname': self.node.name,
                           'ip_addresses': {'private': ['1.2.3.4']},
                           'node': {'node': 'dict'},
                           'state': 'deploying',
                           'uuid': self.node.id},
                          to_dict)
         # States are converted to strings
-        self.assertIsInstance(to_dict['state'], six.string_types)
+        self.assertIsInstance(to_dict['state'], str)
 
     @mock.patch.object(_instance.Instance, 'ip_addresses', autospec=True)
     def test_to_dict_with_allocation(self, mock_ips):
         self.node.provision_state = 'wait call-back'
         self.node.to_dict.return_value = {'node': 'dict'}
-        self.node.instance_info = {'metalsmith_hostname': 'host'}
         mock_ips.return_value = {'private': ['1.2.3.4']}
         self.instance._allocation = mock.Mock()
+        self.instance._allocation.name = 'host'
         self.instance._allocation.to_dict.return_value = {'alloc': 'dict'}
 
         to_dict = self.instance.to_dict()
@@ -157,9 +155,4 @@ class TestInstanceStates(test_provisioner.Base):
                           'uuid': self.node.id},
                          to_dict)
         # States are converted to strings
-        self.assertIsInstance(to_dict['state'], six.string_types)
-
-    def test_deprecated_comparisons(self):
-        for item in _instance.InstanceState:
-            self.assertEqual(item, item.value)
-            self.assertFalse(item != item.value)  # noqa
+        self.assertIsInstance(to_dict['state'], str)
